"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemporalCluster = void 0;
const constructs_1 = require("constructs");
const aws_ec2_1 = require("aws-cdk-lib/aws-ec2");
const aws_rds_1 = require("aws-cdk-lib/aws-rds");
const aws_ecs_1 = require("aws-cdk-lib/aws-ecs");
const TemporalDatastore_1 = require("./TemporalDatastore");
const TemporalVersion_1 = require("./TemporalVersion");
const aws_cdk_lib_1 = require("aws-cdk-lib");
const aws_efs_1 = require("aws-cdk-lib/aws-efs");
const EfsFileResource_1 = require("./customResources/efsFileResource/EfsFileResource");
const TemporalConfiguration_1 = require("./configurations/TemporalConfiguration");
const aws_servicediscovery_1 = require("aws-cdk-lib/aws-servicediscovery");
const ServerServices_1 = require("./services/ServerServices");
const TemporalDatabaseResource_1 = require("./customResources/temporal/TemporalDatabaseResource");
const defaultMachineProperties = {
    cpu: 256,
    memoryLimitMiB: 512,
    cpuArchitecture: aws_ecs_1.CpuArchitecture.X86_64,
};
class TemporalCluster extends constructs_1.Construct {
    constructor(scope, id, clusterProps) {
        super(scope, id);
        this.name = clusterProps.clusterName ?? aws_cdk_lib_1.Names.uniqueId(this);
        this.temporalVersion = clusterProps.temporalVersion ?? TemporalVersion_1.TemporalVersion.LATEST;
        this.temporalConfig = new TemporalConfiguration_1.TemporalConfiguration();
        const servicesProps = this.resolveServiceProps(clusterProps.services);
        this.ecsCluster = this.getOrCreateEcsCluster(clusterProps);
        // FIXME: Add support for mixed datastores configuration (ie. SQL+Cassandra or SQL+ES)
        const datastore = this.getOrCreateDatastore(clusterProps);
        const mainDatabase = new TemporalDatabaseResource_1.TemporalDatabase(this, 'MainDatabase', {
            datastore: datastore,
            databaseName: 'temporal',
            schemaType: 'main',
            removalPolicy: clusterProps.removalPolicy,
        });
        this.temporalConfig.attachDatabase(mainDatabase);
        const visibilityDatabase = new TemporalDatabaseResource_1.TemporalDatabase(this, 'VisibilityDatabase', {
            datastore: datastore,
            databaseName: 'temporal_visibility',
            schemaType: 'visibility',
            removalPolicy: clusterProps.removalPolicy,
        });
        this.temporalConfig.attachDatabase(visibilityDatabase);
        this.configEfs = this.setupConfigFileSystem(clusterProps, this.temporalConfig);
        this.services = {
            // FIXME: Add a config property to determine if must setup an all-in-one container, or a 4 containers cluster
            // single: new SingleService(this, { machine: servicesProps.single.machine }),
            frontend: new ServerServices_1.FrontendService(this, { machine: servicesProps.frontend.machine }),
            matching: new ServerServices_1.MatchingService(this, { machine: servicesProps.matching.machine }),
            history: new ServerServices_1.HistoryService(this, { machine: servicesProps.history.machine }),
            worker: new ServerServices_1.WorkerService(this, { machine: servicesProps.worker.machine }),
            web: servicesProps.web.enabled ? new ServerServices_1.WebService(this, { machine: servicesProps.web.machine }) : undefined,
        };
        this.wireUpNetworkAuthorizations({ main: mainDatabase, visibility: visibilityDatabase });
        if (clusterProps.cloudMapRegistration) {
            this.services.frontend.fargateService.enableCloudMap({
                name: clusterProps.cloudMapRegistration?.serviceName,
                cloudMapNamespace: clusterProps.cloudMapRegistration?.namespace,
                dnsRecordType: aws_servicediscovery_1.DnsRecordType.A,
            });
        }
    }
    // FIXME: Refactor this
    resolveServiceProps(services) {
        var _a;
        const out = {};
        for (const service of ['frontend', 'history', 'matching', 'worker', 'single', 'web']) {
            out[service] = {
                ...services?.defaults,
                ...services?.[service],
                machine: {
                    ...defaultMachineProperties,
                    ...services?.defaults?.machine,
                    ...services?.[service]?.machine,
                },
            };
        }
        (_a = out.web).enabled ?? (_a.enabled = true);
        return out;
    }
    getOrCreateDatastore(props) {
        // FIXME: Validate that requested/provided datastore is supported by requested temporal server version
        // https://docs.temporal.io/docs/server/versions-and-dependencies/#server-versioning-and-support-policy
        if (props.datastore && props.datastoreOptions) {
            throw `You must specify either a datastore or datastoreOptions, not both.`;
        }
        else if (props.datastore) {
            return props.datastore;
        }
        else {
            return new TemporalDatastore_1.AuroraServerlessTemporalDatastore(this, 'Datastore', {
                engine: aws_rds_1.DatabaseClusterEngine.auroraMysql({ version: aws_rds_1.AuroraMysqlEngineVersion.VER_2_10_1 }),
                vpc: props.vpc,
                removalPolicy: props.removalPolicy,
                ...props.datastoreOptions,
            });
        }
    }
    getOrCreateEcsCluster(props) {
        if (props.ecsCluster) {
            return props.ecsCluster;
        }
        else {
            return new aws_ecs_1.Cluster(this, 'EcsCluster', {
                vpc: props.vpc,
                enableFargateCapacityProviders: true,
                containerInsights: true,
            });
        }
    }
    setupConfigFileSystem(props, temporalConfig) {
        const configEfs = new aws_efs_1.FileSystem(this, 'ConfigFS', {
            vpc: props.vpc,
            removalPolicy: props.removalPolicy,
        });
        // Temporal server image builds the main configuration file, from a template file
        // Ideally, we would replace that template file with something that already contains
        // most of our configuration items. However, we faced issues implementing that:
        // posix permissions when the script tries to write new dockerized config file; ECS doesn't
        // allow to map a single file from an EFS volume (only a directory)...
        // At present, we rely on the default template file, and pass all config items through env variables.
        // A possible alternative: https://kichik.com/2020/09/10/mounting-configuration-files-in-fargate/
        // new EfsFile(this, 'TemporalConfig', {
        //     fileSystem: configEfs,
        //     path: `/temporal/config/config_template.yaml`,
        //     vpc: props.vpc,
        //     contents: Lazy.string({ produce: () => temporalConfig.stringifyConfiguration() }),
        // });
        new EfsFileResource_1.EfsFile(this, 'TemporalDynamicConfig', {
            fileSystem: configEfs,
            path: `/temporal/dynamic_config/dynamic_config.yaml`,
            vpc: props.vpc,
            contents: aws_cdk_lib_1.Lazy.string({ produce: () => temporalConfig.stringifyDynamic() }),
        });
        new EfsFileResource_1.EfsFile(this, 'TemporalWebConfig', {
            fileSystem: configEfs,
            path: `/temporal/web_config/web_config.yaml`,
            vpc: props.vpc,
            contents: aws_cdk_lib_1.Lazy.string({ produce: () => temporalConfig.stringifyWeb() }),
        });
        return configEfs;
    }
    wireUpNetworkAuthorizations(databases) {
        const asConnections = (defaultPort, nodeTypes) => {
            return new aws_ec2_1.Connections({
                securityGroups: nodeTypes.flatMap((nodeType) => this.services[nodeType]?.connections?.securityGroups || []),
                defaultPort: defaultPort ? aws_ec2_1.Port.tcp(defaultPort) : undefined,
            });
        };
        const portsConfig = this.temporalConfig.configuration.services;
        // Note that the single machine, if activated, plays all server roles
        const frontendConnections = asConnections(portsConfig.frontend.rpc.grpcPort, ['frontend', 'single']);
        const historyConnections = asConnections(portsConfig.history.rpc.grpcPort, ['history', 'single']);
        const matchingConnections = asConnections(portsConfig.matching.rpc.grpcPort, ['matching', 'single']);
        const workerConnections = asConnections(portsConfig.worker.rpc.grpcPort, ['worker', 'single']);
        const webConnections = asConnections(8080 /* FIXME */, ['web']);
        frontendConnections.allowToDefaultPort(historyConnections);
        frontendConnections.allowToDefaultPort(matchingConnections);
        frontendConnections.allowToDefaultPort(workerConnections);
        frontendConnections.allowToDefaultPort(databases.main.datastore);
        frontendConnections.allowToDefaultPort(databases.visibility.datastore);
        matchingConnections.allowToDefaultPort(frontendConnections);
        matchingConnections.allowToDefaultPort(historyConnections);
        matchingConnections.allowToDefaultPort(databases.main.datastore);
        historyConnections.allowToDefaultPort(matchingConnections);
        historyConnections.allowToDefaultPort(databases.main.datastore);
        workerConnections.allowToDefaultPort(frontendConnections);
        webConnections.allowToDefaultPort(frontendConnections);
        // Allow Ringpop/membership communications between all server nodes, on every membership ports
        const allServerConnections = asConnections(undefined, ['frontend', 'history', 'matching', 'worker', 'single']);
        allServerConnections.allowInternally(aws_ec2_1.Port.tcp(portsConfig.frontend.rpc.membershipPort));
        allServerConnections.allowInternally(aws_ec2_1.Port.tcp(portsConfig.matching.rpc.membershipPort));
        allServerConnections.allowInternally(aws_ec2_1.Port.tcp(portsConfig.history.rpc.membershipPort));
        allServerConnections.allowInternally(aws_ec2_1.Port.tcp(portsConfig.worker.rpc.membershipPort));
        this.rolesConnections = {
            frontend: frontendConnections,
            history: historyConnections,
            matching: matchingConnections,
            worker: workerConnections,
            web: webConnections,
        };
    }
    get connections() {
        return this.rolesConnections.frontend;
    }
}
exports.TemporalCluster = TemporalCluster;
//# sourceMappingURL=TemporalCluster.js.map