"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemporalConfiguration = void 0;
const aws_cdk_lib_1 = require("aws-cdk-lib");
const aws_ecs_1 = require("aws-cdk-lib/aws-ecs");
const yaml_1 = __importDefault(require("yaml"));
class TemporalConfiguration {
    constructor() {
        this.configuration = { ...baseTemporalConfiguration };
        this.dynamic = { ...baseDynamicConfiguration };
        this.web = { ...baseWebConfiguration };
    }
    toEnvironmentVariables() {
        return {
            //
            // See https://github.com/temporalio/temporal/blob/master/docker/config_template.yaml for reference.
            LOG_LEVEL: 'debug,info',
            NUM_HISTORY_SHARDS: `${this.configuration.persistence.numHistoryShards}`,
            ...this.envVariables,
            DYNAMIC_CONFIG_FILE_PATH: '/etc/temporal/dynamic_config/dynamic_config.yaml',
        };
    }
    toSecrets() {
        return this.secrets;
    }
    stringifyConfiguration() {
        return yaml_1.default.stringify(this.configuration);
    }
    stringifyDynamic() {
        return yaml_1.default.stringify(this.dynamic);
    }
    stringifyWeb() {
        return yaml_1.default.stringify(this.web);
    }
    attachDatabase(database) {
        // FIXME: Add support for various types of datastores. At this time, this is much more difficult that it should
        // because of the fact that we can't replace the temporal configuration file directly, and must therefore go
        // through environement variables...
        if (database.datastore.plugin === 'mysql') {
            this.envVariables = {
                DB: 'mysql',
                MYSQL_SEEDS: database.datastore.host,
                DB_PORT: aws_cdk_lib_1.Token.asString(database.datastore.port),
                DBNAME: 'temporal',
                DBNAME_VISIBILITY: 'temporal_visibility',
                // Required for Aurora MySQL 5.7
                // https://github.com/temporalio/temporal/issues/1251
                // https://github.com/temporalio/temporal/blob/71093d7c5baed10546d1e91608ab814f73c6fdba/docker/auto-setup.sh#L157
                MYSQL_TX_ISOLATION_COMPAT: 'true',
            };
            this.secrets = {
                MYSQL_USER: aws_ecs_1.Secret.fromSecretsManager(database.datastore.secret, 'username'),
                MYSQL_PWD: aws_ecs_1.Secret.fromSecretsManager(database.datastore.secret, 'password'),
            };
        }
    }
}
exports.TemporalConfiguration = TemporalConfiguration;
// Note: This configuration will actually be reevaluated at each docker launch, using dockerize.
// It is therefore possible and acceptable to include reference to environment variables in this.
// Note however that
// See https://github.com/temporalio/temporal/blob/master/docker/config_template.yaml for reference.
const baseTemporalConfiguration = {
    log: {
        stdout: true,
        level: 'debug,info',
    },
    persistence: {
        numHistoryShards: 8,
        defaultStore: 'default',
        visibilityStore: 'visibility',
        datastores: {
            default: {},
            visibility: {},
        },
    },
    global: {
        membership: {
            name: 'temporal',
            maxJoinDuration: '30s',
            broadcastAddress: {},
        },
        pprof: {
            port: 7936,
        },
    },
    services: {
        frontend: {
            rpc: {
                grpcPort: 7233,
                membershipPort: 6933,
                bindOnIP: '0.0.0.0',
            },
            metrics: {
                tags: {
                    type: 'frontend',
                },
                prometheus: {
                    timerType: 'histogram',
                    listenAddress: '0.0.0.0:9090',
                },
            },
        },
        history: {
            rpc: {
                grpcPort: 7234,
                membershipPort: 6934,
                bindOnIP: '0.0.0.0',
            },
            metrics: {
                tags: {
                    type: 'history',
                },
                prometheus: {
                    timerType: 'histogram',
                    listenAddress: '0.0.0.0:9090',
                },
            },
        },
        matching: {
            rpc: {
                grpcPort: 7235,
                membershipPort: 6935,
                bindOnIP: '0.0.0.0',
            },
            metrics: {
                tags: {
                    type: 'matching',
                },
                prometheus: {
                    timerType: 'histogram',
                    listenAddress: '0.0.0.0:9090',
                },
            },
        },
        worker: {
            rpc: {
                grpcPort: 7239,
                membershipPort: 6939,
                bindOnIP: '0.0.0.0',
            },
            metrics: {
                tags: {
                    type: 'worker',
                },
                prometheus: {
                    timerType: 'histogram',
                    listenAddress: '0.0.0.0:9090',
                },
            },
        },
    },
    clusterMetadata: {
        enableGlobalDomain: false,
        failoverVersionIncrement: 10,
        masterClusterName: 'active',
        currentClusterName: 'active',
        clusterInformation: {
            active: {
                enabled: true,
                initialFailoverVersion: 1,
                rpcName: 'temporal-frontend',
                rpcAddress: '127.0.0.1:7933',
            },
        },
    },
    dcRedirectionPolicy: {
        policy: 'noop',
        toDC: '',
    },
    archival: {
        status: 'disabled',
    },
    publicClient: {
        hostPort: 'abcdef.mydomain.com-frontend:7233',
    },
    dynamicConfigClient: {
        filepath: '/etc/temporal/dynamic_config/dynamic_config.yaml',
        pollInterval: '10s',
    },
};
const baseDynamicConfiguration = {};
const baseWebConfiguration = {
    auth: {
        enabled: false,
    },
    routing: {
        issue_report_link: 'https://github.com/temporalio/web/issues/new/choose',
    },
};
//# sourceMappingURL=TemporalConfiguration.js.map