"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.onEvent = void 0;
const child_process_1 = require("child_process");
const wait_port_1 = __importDefault(require("wait-port"));
const aws_sdk_1 = require("aws-sdk");
const SecretsManagerClient = new aws_sdk_1.SecretsManager({});
async function onEvent(event) {
    const resourceProps = await extractResourceProperties(event);
    switch (event.RequestType) {
        case 'Create':
        case 'Update':
            waitForDatabase(resourceProps);
            createDatabase(resourceProps);
            upgradeDatabase(resourceProps);
            return {
                PhysicalResourceId: resourceProps.resourcePhysicalId,
            };
        case 'Delete': {
            waitForDatabase(resourceProps);
            dropDatabase(resourceProps);
            return {
                PhysicalResourceId: resourceProps.resourcePhysicalId,
            };
        }
    }
}
exports.onEvent = onEvent;
async function extractResourceProperties(event) {
    const inputProps = event.ResourceProperties;
    if (!inputProps.DatastorePlugin)
        throw new Error('"DatastorePlugin" is required');
    if (!['mysql', 'postgres', 'cassandra', 'elasticsearch'].includes(inputProps.DatastorePlugin))
        throw new Error('"DatastorePlugin" value bust be one of "mysql", "postgres", "cassandra" or "elasticsearch"');
    if (!inputProps.DatastoreHost)
        throw new Error('"DatastoreHost" is required');
    if (!inputProps.DatastorePort)
        throw new Error('"DatastorePort" is required');
    if (!inputProps.DatastorePort.match(/^[1-9][0-9]+$/))
        throw new Error('"DatastorePort" must be a number');
    if (!inputProps.DatastoreSecretId)
        throw new Error('"DatastoreSecretId" is required');
    if (!inputProps.DatabaseName)
        throw new Error('"DatabaseName" is required');
    if (!inputProps.SchemaType)
        throw new Error('"SchemaType" is required');
    if (!['main', 'visibility'].includes(inputProps.SchemaType))
        throw new Error('"SchemaType" must be either "main" or "visibility"');
    if (!inputProps.TemporalVersion)
        throw new Error('"TemporalVersion" is required');
    const datastoreSecret = await SecretsManagerClient.getSecretValue({
        SecretId: inputProps.DatastoreSecretId,
    }).promise();
    const datastoreSecretObject = JSON.parse(datastoreSecret.SecretString);
    const resourcePhysicalId = `${inputProps.DatastorePlugin}://${inputProps.DatastoreHost}:${inputProps.DatastorePort}/${inputProps.DatabaseName}`;
    return {
        datastorePlugin: inputProps.DatastorePlugin,
        datastoreHost: inputProps.DatastoreHost,
        datastorePort: parseInt(inputProps.DatastorePort),
        datastoreUser: datastoreSecretObject.username,
        datastorePassword: datastoreSecretObject.password,
        databaseName: inputProps.DatabaseName,
        schemaType: inputProps.SchemaType,
        temporalVersion: inputProps.TemporalVersion,
        resourcePhysicalId,
    };
}
function waitForDatabase(resourceProps) {
    (0, wait_port_1.default)({
        host: resourceProps.datastoreHost,
        port: resourceProps.datastorePort,
    });
}
function createDatabase(resourceProps) {
    switch (resourceProps.datastorePlugin) {
        case 'mysql':
        case 'postgres':
            execTemporalSqlTool(resourceProps, ['create-database', '--database', resourceProps.databaseName]);
            execTemporalSqlTool(resourceProps, ['setup-schema', '-v', '0.0']);
            break;
        case 'cassandra':
            throw new Error('createDatabase(cassandra) is not yet implemented');
        case 'elasticsearch':
            throw new Error('createDatabase(elasticsearch) is not yet implemented');
    }
}
function upgradeDatabase(resourceProps) {
    const type = resourceProps.schemaType === 'main' ? 'temporal' : 'visibility';
    switch (resourceProps.datastorePlugin) {
        case 'mysql': {
            const schemaDir = `/opt/temporal/schema/mysql/v57/${type}/versioned`;
            execTemporalSqlTool(resourceProps, ['update-schema', '-d', schemaDir]);
            break;
        }
        case 'postgres': {
            const schemaDir = `/opt/temporal/schema/postgresql/v96/${type}/versioned`;
            execTemporalSqlTool(resourceProps, ['update-schema', '-d', schemaDir]);
            break;
        }
        case 'cassandra': {
            // const schemaDir = `/opt/temporal/schema/cassandra/${type}/versioned`;
            throw new Error('upgradeDatabase(cassandra) is not yet implemented');
        }
        case 'elasticsearch': {
            // const schemaDir = `/opt/temporal/schema/elasticsearch/visibility/versioned`;
            throw new Error('upgradeDatabase(elasticsearch) is not yet implemented');
        }
    }
}
function dropDatabase(resourceProps) {
    switch (resourceProps.datastorePlugin) {
        case 'mysql':
        case 'postgres':
            execTemporalSqlTool(resourceProps, ['drop-database', '--database', resourceProps.databaseName, '--force']);
            break;
        case 'cassandra':
            throw new Error('dropDatabase(cassandra) is not yet implemented');
        case 'elasticsearch':
            throw new Error('dropDatabase(elasticsearch) is not yet implemented');
    }
}
function execTemporalSqlTool(context, command) {
    const args = [];
    const env = {};
    args.push('--endpoint', context.datastoreHost);
    args.push('--port', `${context.datastorePort}`);
    args.push('--user', context.datastoreUser);
    env['SQL_PASSWORD'] = context.datastorePassword;
    args.push('--database', context.databaseName);
    // Tx Attributes
    if (context.datastorePlugin === 'mysql') {
        args.push('--connect-attributes', 'tx_isolation=READ-COMMITTED');
    }
    (0, child_process_1.execFileSync)('/opt/temporal/bin/temporal-sql-tool', [...args, ...command], {
        encoding: 'utf-8',
        env,
    });
}
//# sourceMappingURL=TemporalDatabaseHandler.js.map